import threading
import time
import serial
import sys
import pygame

# FeederThread continuosly reads the data coming from the MCU
# Data from the MCU reports three temperature values and two constants:
# temp_avg = raw temperature
# trans = predictive filter result
# low = low pass filter result
# a and filterTau are the predictive and low pass filter constants 
#
# temp_avg trans low a filterTau
class FeederThread(threading.Thread):
  running = True
  addPoint = None
  
  def getReading(self):
    x = self.s.readline()
    k = x.split()
    if len(k)<5:
      print "yuck, got", x
      return self.getReading()

    try:
      return (float(k[0]),float(k[1]),float(k[2]),float(k[3]),float(k[4]))
    except ValueError:
      print "yuck, got ", x
      return self.getReading()  

  def run(self):
    self.s = serial.Serial("/dev/ttyUSB0", 115200)
    self.s.readline()
    
    while self.running: 
      if self.addPoint:
        self.addPoint(self.getReading())


class Plotter:
  bgcolor = (255,255,255)
  linecolors = [(0,0,255),(0,255,0),(255,0,0),(0,0,0)]
  linewidth = 3
  textcolor = (0,0,0)
  textsize = 30
  numInputs = 5 #total number of inputs coming from the MCU (also the width of the history array)
  smallFontSize = 16

  def handleEvents(self):
    for event in pygame.event.get():
      if event.type is pygame.QUIT:
        self.feeder.running = False
        sys.exit()
      elif event.type is pygame.MOUSEBUTTONDOWN:
        self.clearHistory()

  def clearHistory(self):
    self.history = [ [0.0 for x in xrange(self.width)] for y in xrange(self.numInputs) ]

  def generatePoints(self):
    history = [self.history[0], self.history[2]] # grab the colums of the history array that contian the lines you wish to graph
    #history = [self.history[0]]

    return [[(i, self.verticalTransform(line[i])) for i in xrange(len(line))] for line in history ]

  def addPoint(self, q):
    if self.history:
      for i in range(self.numInputs):
        self.history[i].pop(0)
        self.history[i].append(q[i])

  def __init__(self, feeder):
    self.size = self.width, self.height = 640,480
    self.feeder = feeder
    self.clearHistory()
    self.feeder.addPoint = self.addPoint

  def calculateVerticalScale(self):
    self.top = max([max(self.history[0]),max(self.history[2])])
    self.bot = min([min(self.history[0]),min(self.history[2])])
    rg = (self.top-self.bot) * 1.2
    
    if rg > 0:
      self.vscale = self.height / rg
      self.vzero = int(self.height *((self.top-0)/(self.top-self.bot) + 0.1)/1.2)
    else:
      self.vscale = 1
      self.vzero = self.height / 2


  def verticalTransform(self, q):
    return self.vzero - self.vscale * q

  ### Main init code
  def main(self):
    # set up display
    pygame.init()
    pygame.font.init()
    screen = pygame.display.set_mode(self.size)

    # set up feeder
    self.feeder.start()
    
    background = pygame.Surface(self.size).convert_alpha()
    background.fill(self.bgcolor)
    #background = pygame.image.load("background.jpg").convert_alpha()
    
    font = pygame.font.SysFont("Verdana", self.textsize)
    smallFont = pygame.font.SysFont("Verdana", self.smallFontSize)
 
    k = 0
    while True:
      # set up axes
      self.calculateVerticalScale()
      
      # draw stuff to screen
      #   background
      screen.blit(background, (0,0))
      #   data points
      for color,points in zip(self.linecolors,self.generatePoints()):
        pygame.draw.lines(screen, color, False, points, self.linewidth)
            
      #draw scale
      for i in range(int(self.bot*1.2)-int(self.bot*1.2)%10,int(self.top*1.2)-int(self.top*1.2)%10,10):
        #print self.bot , self.top
        scalePos = self.verticalTransform(i)
        pygame.draw.line(screen,self.textcolor,(0,scalePos),(self.width,scalePos))
        scaleNumber = smallFont.render(str(i),True,self.textcolor)
        numberSize = smallFont.size(str(i))
        screen.blit(scaleNumber,(self.width - numberSize[0] - 2, scalePos-numberSize[1]-1))

      #draw current
      teNumber = str(self.history[0][-1]) 
      text = smallFont.render(teNumber, True, self.textcolor)
      numberSize = smallFont.size(teNumber)
      screen.blit(text, (self.width - numberSize[0] -2, self.verticalTransform(float(teNumber)) + 5 ) )
      #   text

      teNumber = "%.2f" % (sum(self.history[2][-10:-1])/len(self.history[2][-10:-1]))
      text = smallFont.render(teNumber, True, self.textcolor)
      numberSize = smallFont.size(teNumber)
      screen.blit(text, (self.width - numberSize[0] -2, self.verticalTransform(float(teNumber)) - numberSize[1]-2 ) )

      content = "Current Temp: %.2f degrees" % self.history[0][-1]
      text = font.render(content, True, self.textcolor)
      textsize0 = font.size(content)
      screen.blit(text, ((self.width - textsize0[0])/2,10))
      #
      content = "Current Estimate: %.2f degrees" % self.history[2][-1]
      text = font.render(content, True, self.textcolor)
      textsize1 = font.size(content)
      screen.blit(text, ((self.width - textsize1[0])/2,10 + textsize0[1] + 10))

#
#      content = "Click anywhere to clear history!"
#      text = font.render(content, True, self.textcolor)
#      textsize2 = font.size(content)
#      screen.blit(text, ((self.width - textsize2[0])/2,self.height-textsize2[1]-10))
#      
      # done drawing... doublebuffer!
      pygame.display.flip()
      
      self.handleEvents()

    
if __name__ == "__main__":
  feeder = FeederThread()
  plotter = Plotter(feeder)
  plotter.main()
